// --- [adxl.h] ---------------------------------------------------------------

#ifndef _ADXL34X_H_
#define _ADXL34X_H_

#define ADXL_TWI_ADR				0x1d * 2 //  58 
#define ADXL_TWI_ALT_ADR		0x53 * 2 // 166
#define ADXL_ID					0xE5

#define ADXL_DEVID				0x00
#define ADXL_THRESH_TAP			0x1D
#define ADXL_OFSX					0x1E
#define ADXL_OFSY					0x1F
#define ADXL_OFSZ					0x20
#define ADXL_TAP_DUR				0x21
#define ADXL_LATENT				0x22
#define ADXL_WINDOW				0x23
#define ADXL_THRESH_ACT			0x24
#define ADXL_THRESH_INACT		0x25
#define ADXL_TIME_INACT			0x26
#define ADXL_ACT_INACT_CTL		0x27
#define ADXL_THRESH_FF			0x28
#define ADXL_TIME_FF				0x29
#define ADXL_TAP_AXES			0x2A
#define ADXL_ACT_TAP_STATUS	0x2B
#define ADXL_BW_RATE				0x2C
#define ADXL_POWER_CTL			0x2D
#define ADXL_INT_ENABLE			0x2E
#define ADXL_INT_MAP				0x2F
#define ADXL_INT_SOURCE			0x30
#define ADXL_DATA_FORMAT		0x31
#define ADXL_DATA					0x32
#define ADXL_FIFO_CTL			0x38
#define ADXL_FIFO_STATUS		0x39


// ADXL_BW_RATE
#define BW_RATE_LOW_POWER		0x10
#define BW_RATE_6_25				0x06
#define BW_RATE_12_5				0x07
#define BW_RATE_25				0x08
#define BW_RATE_50				0x09
#define BW_RATE_100				0x0A
#define BW_RATE_200				0x0B
#define BW_RATE_400				0x0C
#define BW_RATE_800				0x0D
#define BW_RATE_1600				0x0E
#define BW_RATE_3200				0x0F

// ADXL_FIFO_CTL
#define FIFO_CTL_FIFO_BYPASS	0x00
#define FIFO_CTL_FIFO_FIFO		0x40
#define FIFO_CTL_FIFO_STREAM	0x80
#define FIFO_CTL_FIFO_TRIGGER	0xC0
#define FIFO_CTL_TRIGGER		0x20

// ADXL_POWER_CTL
#define POWER_CTL_WAKEUP1		0x03
#define POWER_CTL_WAKEUP2		0x02
#define POWER_CTL_WAKEUP4		0x01
#define POWER_CTL_WAKEUP8		0x00
#define POWER_CTL_SLEEP			0x04
#define POWER_CTL_MEASURE		0x08
#define POWER_CTL_AUTO_SLEEP	0x10
#define POWER_CTL_LINK			0x20

// ADXL_INT_ENABLE/MAP/SOURCE
#define INT_OVERRUN				0x01
#define INT_WATERMARK			0x02
#define INT_FREEFALL				0x04
#define INT_INACTIVITY			0x08
#define INT_ACTIVITY				0x10
#define INT_DOUBLE_TAP			0x20
#define INT_SINGLE_TAP			0x40
#define INT_DATA_READY			0x80

// ADXL_DATA_FORMAT
#define DATA_2G					0x00
#define DATA_4G					0x01
#define DATA_8G					0x02
#define DATA_16G					0x03
#define DATA_JUSTIFY				0x04
#define DATA_FULL_RES			0x08
#define DATA_INT_INVERT			0x20
#define DATA_SPI					0x40
#define DATA_SELFTEST			0x80
#endif

#ifdef DO_NOT_TRANSLATE
struct adxl34x_platform_data {

	/*
	 * X,Y,Z Axis Offset:
	 * offer user offset adjustments in twoscompliment
	 * form with a scale factor of 15.6 mg/LSB (i.e. 0x7F = +2 g)
	 */

	char x_axis_offset;
	char y_axis_offset;
	char z_axis_offset;

	/*
	 * TAP_X/Y/Z Enable: Setting TAP_X, Y, or Z Enable enables X,
	 * Y, or Z participation in Tap detection. A '0' excludes the
	 * selected axis from participation in Tap detection.
	 * Setting the SUPPRESS bit suppresses Double Tap detection if
	 * acceleration greater than tap_threshold is present between
	 * taps.
	 */

#define ADXL_SUPPRESS 	(1 << 3)
#define ADXL_TAP_X_EN 	(1 << 2)
#define ADXL_TAP_Y_EN 	(1 << 1)
#define ADXL_TAP_Z_EN 	(1 << 0)

	unsigned char tap_axis_control;

	/*
	 * tap_threshold:
	 * holds the threshold value for tap detection/interrupts.
	 * The data format is unsigned. The scale factor is 62.5 mg/LSB
	 * (i.e. 0xFF = +16 g). A zero value may result in undesirable
	 * behavior if Tap/Double Tap is enabled.
	 */

	unsigned char tap_threshold;

	/*
	 * tap_duration:
	 * is an unsigned time value representing the maximum
	 * time that an event must be above the tap_threshold threshold
	 * to qualify as a tap event. The scale factor is 625 us/LSB. A zero
	 * value will prevent Tap/Double Tap functions from working.
	 */

	unsigned char tap_duration;

	/*
	 * tap_latency:
	 * is an unsigned time value representing the wait time
	 * from the detection of a tap event to the opening of the time
	 * window tap_window for a possible second tap event. The scale
	 * factor is 1.25 ms/LSB. A zero value will disable the Double Tap
	 * function.
	 */

	unsigned char tap_latency;

	/*
	 * tap_window:
	 * is an unsigned time value representing the amount
	 * of time after the expiration of tap_latency during which a second
	 * tap can begin. The scale factor is 1.25 ms/LSB. A zero value will
	 * disable the Double Tap function.
	 */

	unsigned char tap_window;

	/*
	 * act_axis_control:
	 * X/Y/Z Enable: A '1' enables X, Y, or Z participation in activity
	 * or inactivity detection. A '0' excludes the selected axis from
	 * participation. If all of the axes are excluded, the function is
	 * disabled.
	 * AC/DC: A '0' = DC coupled operation and a '1' = AC coupled
	 * operation. In DC coupled operation, the current acceleration is
	 * compared with activity_threshold and inactivity_threshold directly
	 * to determine whether activity or inactivity is detected. In AC
	 * coupled operation for activity detection, the acceleration value
	 * at the start of activity detection is taken as a reference value.
	 * New samples of acceleration are then compared to this
	 * reference value and if the magnitude of the difference exceeds
	 * activity_threshold the device will trigger an activity interrupt. In
	 * AC coupled operation for inactivity detection, a reference value
	 * is used again for comparison and is updated whenever the
	 * device exceeds the inactivity threshold. Once the reference
	 * value is selected, the device compares the magnitude of the
	 * difference between the reference value and the current
	 * acceleration with inactivity_threshold. If the difference is below
	 * inactivity_threshold for a total of inactivity_time, the device is
	 * considered inactive and the inactivity interrupt is triggered.
	 */

#define ADXL_ACT_ACDC   	(1 << 7)
#define ADXL_ACT_X_EN   	(1 << 6)
#define ADXL_ACT_Y_EN   	(1 << 5)
#define ADXL_ACT_Z_EN   	(1 << 4)
#define ADXL_INACT_ACDC 	(1 << 3)
#define ADXL_INACT_X_EN 	(1 << 2)
#define ADXL_INACT_Y_EN 	(1 << 1)
#define ADXL_INACT_Z_EN 	(1 << 0)

	unsigned char act_axis_control;

	/*
	 * activity_threshold:
	 * holds the threshold value for activity detection.
	 * The data format is unsigned. The scale factor is
	 * 62.5 mg/LSB. A zero value may result in undesirable behavior if
	 * Activity interrupt is enabled.
	 */

	unsigned char activity_threshold;

	/*
	 * inactivity_threshold:
	 * holds the threshold value for inactivity
	 * detection. The data format is unsigned. The scale
	 * factor is 62.5 mg/LSB. A zero value may result in undesirable
	 * behavior if Inactivity interrupt is enabled.
	 */

	unsigned char inactivity_threshold;

	/*
	 * inactivity_time:
	 * is an unsigned time value representing the
	 * amount of time that acceleration must be below the value in
	 * inactivity_threshold for inactivity to be declared. The scale factor
	 * is 1 second/LSB. Unlike the other interrupt functions, which
	 * operate on unfiltered data, the inactivity function operates on the
	 * filtered output data. At least one output sample must be
	 * generated for the inactivity interrupt to be triggered. This will
	 * result in the function appearing un-responsive if the
	 * inactivity_time register is set with a value less than the time
	 * constant of the Output Data Rate. A zero value will result in an
	 * interrupt when the output data is below inactivity_threshold.
	 */

	unsigned char inactivity_time;

	/*
	 * free_fall_threshold:
	 * holds the threshold value for Free-Fall detection.
	 * The data format is unsigned. The root-sum-square(RSS) value
	 * of all axes is calculated and compared to the value in
	 * free_fall_threshold to determine if a free fall event may be
	 * occurring.  The scale factor is 62.5 mg/LSB. A zero value may
	 * result in undesirable behavior if Free-Fall interrupt is
	 * enabled. Values between 300 and 600 mg (0x05 to 0x09) are
	 * recommended.
	 */

	unsigned char free_fall_threshold;

	/*
	 * free_fall_time:
	 * is an unsigned time value representing the minimum
	 * time that the RSS value of all axes must be less than
	 * free_fall_threshold to generate a Free-Fall interrupt. The
	 * scale factor is 5 ms/LSB. A zero value may result in
	 * undesirable behavior if Free-Fall interrupt is enabled.
	 * Values between 100 to 350 ms (0x14 to 0x46) are recommended.
	 */

	unsigned char free_fall_time;

	/*
	 * data_rate:
	 * Selects device bandwidth and output data rate.
	 * RATE = 3200 Hz / (2^(15 - x)). Default value is 0x0A, or 100 Hz
	 * Output Data Rate. An Output Data Rate should be selected that
	 * is appropriate for the communication protocol and frequency
	 * selected. Selecting too high of an Output Data Rate with a low
	 * communication speed will result in samples being discarded.
	 */

	unsigned char data_rate;

	/*
	 * data_range:
	 * FULL_RES: When this bit is set with the device is
	 * in Full-Resolution Mode, where the output resolution increases
	 * with RANGE to maintain a 4 mg/LSB scale factor. When this
	 * bit is cleared the device is in 10-bit Mode and RANGE determine the
	 * maximum g-Range and scale factor.
	 */

#define ADXL_FULL_RES    	(1 << 3)
#define ADXL_RANGE_PM_2g	0
#define ADXL_RANGE_PM_4g	1
#define ADXL_RANGE_PM_8g	2
#define ADXL_RANGE_PM_16g	3

	unsigned char data_range;

	/*
	 * low_power_mode:
	 * A '0' = Normal operation and a '1' = Reduced
	 * power operation with somewhat higher noise.
	 */

	unsigned char low_power_mode;

	/*
	 * power_mode:
	 * LINK: A '1' with both the activity and inactivity functions
	 * enabled will delay the start of the activity function until
	 * inactivity is detected. Once activity is detected, inactivity
	 * detection will begin and prevent the detection of activity. This
	 * bit serially links the activity and inactivity functions. When '0'
	 * the inactivity and activity functions are concurrent. Additional
	 * information can be found in the Application section under Link
	 * Mode.
	 * AUTO_SLEEP: A '1' sets the ADXL34x to switch to Sleep Mode
	 * when inactivity (acceleration has been below inactivity_threshold
	 * for at least inactivity_time) is detected and the LINK bit is set.
	 * A '0' disables automatic switching to Sleep Mode. See SLEEP
	 * for further description.
	 */

#define ADXL_LINK    	(1 << 5)
#define ADXL_AUTO_SLEEP (1 << 4)

	unsigned char power_mode;

	/*
	 * fifo_mode:
	 * BYPASS The FIFO is bypassed
	 * FIFO   FIFO collects up to 32 values then stops collecting data
	 * STREAM FIFO holds the last 32 data values. Once full, the FIFO's
	 * 	  oldest data is lost as it is replaced with newer data
	 *
	 * DEFAULT should be ADXL_FIFO_STREAM
	 */

#define ADXL_FIFO_BYPASS	0
#define ADXL_FIFO_FIFO		1
#define ADXL_FIFO_STREAM	2

	unsigned char fifo_mode;

	/*
	 * watermark:
	 * The Watermark feature can be used to reduce the interrupt load
	 * of the system. The FIFO fills up to the value stored in watermark
	 * [1..32] and then generates an interrupt.
	 * A '0' disables the watermark feature.
	 */

	unsigned char watermark;

	unsigned int ev_type;	/* EV_ABS or EV_REL */

	unsigned int ev_code_x;	/* ABS_X,Y,Z or REL_X,Y,Z */
	unsigned int ev_code_y;	/* ABS_X,Y,Z or REL_X,Y,Z */
	unsigned int ev_code_z;	/* ABS_X,Y,Z or REL_X,Y,Z */

	/*
	 * A valid BTN or KEY Code; use tap_axis_control to disable
	 * event reporting
	 */

	unsigned int ev_code_tap_x;	/* EV_KEY */
	unsigned int ev_code_tap_y;	/* EV_KEY */
	unsigned int ev_code_tap_z;	/* EV_KEY */

	/*
	 * A valid BTN or KEY Code for Free-Fall or Activity enables
	 * input event reporting. A '0' disables the Free-Fall or
	 * Activity reporting.
	 */

	unsigned int ev_code_ff;	/* EV_KEY */
	unsigned int ev_code_act_inactivity;	/* EV_KEY */

	unsigned char use_int2;
};




static const struct adxl34x_platform_data adxl34x_info = {
	.x_axis_offset = 0,
	.y_axis_offset = 0,
	.z_axis_offset = 0,
	.tap_threshold = 0x31,
	.tap_duration = 0x10,
	.tap_latency = 0x60,
	.tap_window = 0xF0,
	.tap_axis_control = ADXL_TAP_X_EN | ADXL_TAP_Y_EN | ADXL_TAP_Z_EN,
	.act_axis_control = 0xFF,
	.activity_threshold = 5,
	.inactivity_threshold = 3,
	.inactivity_time = 4,
	.free_fall_threshold = 0x7,
	.free_fall_time = 0x20,
	.data_rate = 0x8,
	.data_range = ADXL_FULL_RES,

	.ev_type = EV_ABS,
	.ev_code_x = ABS_X,		/* EV_REL */
	.ev_code_y = ABS_Y,		/* EV_REL */
	.ev_code_z = ABS_Z,		/* EV_REL */

	.ev_code_tap_x = BTN_TOUCH,		/* EV_KEY */
	.ev_code_tap_y = BTN_TOUCH,		/* EV_KEY */
	.ev_code_tap_z = BTN_TOUCH,		/* EV_KEY */

/*	.ev_code_ff = KEY_F,*/		/* EV_KEY */
/*	.ev_code_act_inactivity = KEY_A,*/	/* EV_KEY */
	.power_mode = ADXL_AUTO_SLEEP | ADXL_LINK,
	.fifo_mode = ADXL_FIFO_STREAM,
};



static const struct adxl34x_platform_data adxl34x_info = {
	.x_axis_offset = 0,
	.y_axis_offset = 0,
	.z_axis_offset = 0,
	.tap_threshold = 0x31,
	.tap_duration = 0x10,
	.tap_latency = 0x60,
	.tap_window = 0xF0,
	.tap_axis_control = ADXL_TAP_X_EN | ADXL_TAP_Y_EN | ADXL_TAP_Z_EN,
	.act_axis_control = 0xFF,
	.activity_threshold = 5,
	.inactivity_threshold = 3,
	.inactivity_time = 4,
	.free_fall_threshold = 0x7,
	.free_fall_time = 0x20,
	.data_rate = 0x8,
	.data_range = ADXL_FULL_RES,

	.ev_type = EV_ABS,
	.ev_code_x = ABS_X,		/* EV_REL */
	.ev_code_y = ABS_Y,		/* EV_REL */
	.ev_code_z = ABS_Z,		/* EV_REL */

	.ev_code_tap_x = BTN_TOUCH,		/* EV_KEY */
	.ev_code_tap_y = BTN_TOUCH,		/* EV_KEY */
	.ev_code_tap_z = BTN_TOUCH,		/* EV_KEY */

/*	.ev_code_ff = KEY_F,*/		/* EV_KEY */
/*	.ev_code_act_inactivity = KEY_A,*/	/* EV_KEY */
	.power_mode = ADXL_AUTO_SLEEP | ADXL_LINK,
	.fifo_mode = ADXL_FIFO_STREAM,
};
#endif

// --- [eof] ------------------------------------------------------------------



